/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Logger;
import nl.cloudfarming.client.fleet.machine.MachineDataService.StepInfo;
import nl.cloudfarming.client.fleet.model.MachineProperty;
import nl.cloudfarming.client.fleet.model.Vehicle;
import org.openide.WizardDescriptor;

/**
 * An optional extra panel which can be added to the machine wizard.
 *
 * @author Wytse Visser
 */
class MachineDynamicWizardPanel extends MachineWizardPanel {

    private StepInfo stepInfo;
    private boolean propertiesAdded = false;
    private MachineDynamicVisualPanel visualPanel;
    private List<MachineProperty> properties;
    public static final String PROP_DYNAMIC_PROPMAP = "dynamic properties map";
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.vehicle.wizard");

    /**
     * Creates the MachineDynamicWizardPanel. The content of this panel is based on the
     * given StepInfo object.
     *
     * @param stepInfo The StepInfo with the name and properties for this panel.
     */
    public MachineDynamicWizardPanel(StepInfo stepInfo) {
        this.stepInfo = stepInfo;
    }

    @Override
    protected MachineDynamicVisualPanel getVisual() {
        if (visualPanel == null) {
            visualPanel = new MachineDynamicVisualPanel();
            visualPanel.setName(stepInfo.getName());
        }
        return visualPanel;
    }

    @Override
    public void readSettings(WizardDescriptor wiz) {
        setTitle(wiz.getProperty(Vehicle.PROP_NAME).toString());

        if (!propertiesAdded) {
            properties = stepInfo.getProperties();
            for (MachineProperty property : properties) {
                try {
                    getVisual().addPropertyField(property);
                } catch (IllegalAccessException | InvocationTargetException ex) {
                    LOGGER.warning("Could not add property to visual panel");
                }
            }
            propertiesAdded = true;
        }
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        Map<MachineDynamicWizardPanel, List<MachineProperty>> map = getDynamicPropertyMap(wiz);
        map.put(this, properties);
        wiz.putProperty(PROP_DYNAMIC_PROPMAP, map);
    }

    /**
     * Returns the map with dynamic properties of the given WizardDescriptor.
     * Creates a new one if none exists.
     *
     * @param wiz The WizardDescriptor to get the map from
     * @return The dynamic property map
     */
    public static Map<MachineDynamicWizardPanel, List<MachineProperty>> getDynamicPropertyMap(WizardDescriptor wiz) {
        Map<MachineDynamicWizardPanel, List<MachineProperty>> map;
        if (wiz.getProperty(PROP_DYNAMIC_PROPMAP) == null) {
            map = new HashMap<>();
        } else {
            map = (Map<MachineDynamicWizardPanel, List<MachineProperty>>) wiz.getProperty(PROP_DYNAMIC_PROPMAP);
        }
        return map;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final MachineDynamicWizardPanel other = (MachineDynamicWizardPanel) obj;
        if (!Objects.equals(getVisual().getName(), other.getVisual().getName())) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        return hash;
    }
}
