/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.util.*;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.fleet.machine.MachineDataService;
import nl.cloudfarming.client.fleet.machine.MachineDataService.Model;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineProvider;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * The panel for the initial step in the machine creation wizard.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({
    "emptyNameError=Name can not be empty",
    "nameExistsError=Name already exists",
    "emptyTypeError=No type selected",
    "emptyBrandError=No brand selected",
    "emptyModelError=No model selected",
    "customValue=Custom"
})
public class MachineWizardStartPanel extends MachineWizardPanel {

    private MachineVisualStartPanel visualPanel;
    private final Set<ChangeListener> listeners = new HashSet<>(1);
    private MachineWizardFactory factory;
    private WizardDescriptor wizardDescriptor;
    private CustomItemString customValue = new CustomItemString();

    public MachineWizardStartPanel(MachineWizardFactory factory) {
        this.factory = factory;
    }

    @Override
    protected JComponent createComponent() {
        addChangeListener(new TitleUpdater());
        return super.createComponent();
    }

    @Override
    protected MachineVisualStartPanel getVisual() {
        if (visualPanel == null) {
            visualPanel = new MachineVisualStartPanel(this);
            ChangeListener selectionUpdater = new ComboBoxUpdater();
            addChangeListener(selectionUpdater);
            selectionUpdater.stateChanged(null);
        }
        return visualPanel;
    }

    @Override
    public boolean isValid() {
        String errorMsg = null;
        try {
            validate();
        } catch (WizardValidationException e) {
            errorMsg = e.getLocalizedMessage();
        } finally {
            wizardDescriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, errorMsg);
            return errorMsg == null;
        }
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(WizardDescriptor wiz) {
        this.wizardDescriptor = wiz;
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        wiz.putProperty(Machine.PROP_NAME, getVisual().getNameValue());
        wiz.putProperty(MachineWizardIterator.PROP_MODEL, getSelectedModel());
        wiz.putProperty(Machine.PROP_YEAR, getVisual().getYearValue());
        wiz.putProperty(Machine.PROP_PURCHASEDATE, getVisual().getPurchaseDateValue());
        wiz.putProperty(Machine.PROP_PURCHASEPRICE, getVisual().getPurchasePriceValue());
        wiz.putProperty(Machine.PROP_MILEAGE, getVisual().getMileageValue());
    }

    private Model getSelectedModel() {
        ModelIterator iterator = new ModelIterator();
        Model selectedModel = new Model(getTypeValue(), getBrandValue(), getModelNumberValue());
        while (iterator.hasNext()) {
            Model model = iterator.next();
            if (model.equals(selectedModel)) {
                return model;
            }
        }
        return selectedModel;
    }

    private String getTypeValue() {
        return getVisual().getTypeValue();
    }

    private String getBrandValue() {
        String brand = getVisual().getBrandValue();
        if (customValue.isCustomValue(brand)) {
            brand = getVisual().getCustomBrandValue();
        }
        return brand;
    }

    private String getModelNumberValue() {
        String modelNumber = getVisual().getModelValue();
        if (customValue.isCustomValue(modelNumber)) {
            modelNumber = getVisual().getCustomModelValue();
        }
        return modelNumber;
    }

    private void validate() throws WizardValidationException {
        if (getTypeValue() == null) {
            throw new WizardValidationException(null, Bundle.emptyTypeError(), null);
        }
        if (getBrandValue() == null || getBrandValue().isEmpty()) {
            throw new WizardValidationException(null, Bundle.emptyBrandError(), null);
        }
        if (getModelNumberValue() == null || getModelNumberValue().isEmpty()) {
            throw new WizardValidationException(null, Bundle.emptyModelError(), null);
        }
        validateName(getVisual().getNameValue());
    }

    private void validateName(String name) throws WizardValidationException {
        if (name.equals("")) {
            throw new WizardValidationException(null, Bundle.emptyNameError(), null);
        } else if (machineExists(name)) {
            throw new WizardValidationException(null, Bundle.nameExistsError(), null);
        }
    }

    private boolean machineExists(String name) {
        //FIXME move to vehicle/implement specific wizards
        MachineProvider machineProvider = (MachineProvider) Lookup.getDefault().lookup(factory.getMachineProviderClass());
        assert machineProvider != null;
        
        List<Machine> machines = machineProvider.findAll(factory.getProject());
        for(Machine machine: machines) {
            if(machine.getName().equalsIgnoreCase(name)) {
                return true;
            }
        }
        return false;
    }

    private class TitleUpdater implements ChangeListener {

        @Override
        public void stateChanged(ChangeEvent e) {
            setTitle(getVisual().getNameValue());
        }
    }

    /**
     * This class adds items to the type, brand and model number combo box. Non
     * selectable items are automatically removed. (For instance, if Brand A
     * only has tractors and the user selects the Self propelled sprayer type,
     * Brand A is removed from the brand combo box.)
     */
    private class ComboBoxUpdater implements ChangeListener {

        private ModelIterator modelIterator;
        private String selectedType, selectedBrand, selectedModelNumber;

        @Override
        public void stateChanged(ChangeEvent e) {
            selectedType = getVisual().getTypeValue();
            selectedBrand = getVisual().getBrandValue();
            selectedModelNumber = getVisual().getModelValue();

            updateTypes();
            updateBrands();
            updateModelNumbers();
            toggleCustomInputFields();
        }

        private void updateTypes() {
            Set<String> types = new TreeSet<>();
            modelIterator = new ModelIterator();
            while (modelIterator.hasNext()) {
                Model model = modelIterator.next();
                if (typeSelectable(model)) {
                    types.add(model.getType());
                }
            }
            if (types.size() > 1) {
                types.add("");
            }

            getVisual().setTypes(types.toArray());
        }

        private boolean typeSelectable(Model model) {
            boolean brandValid = selectedBrand == null || customValue.isCustomValue(selectedBrand) || selectedBrand.equals(model.getBrand());
            boolean modelValid = selectedModelNumber == null || customValue.isCustomValue(selectedModelNumber) || selectedModelNumber.equals(model.getModelNumber());

            return brandValid && modelValid;
        }

        private void updateBrands() {
            Set<Object> brands = new TreeSet<>();
            modelIterator = new ModelIterator();
            while (modelIterator.hasNext()) {
                Model model = modelIterator.next();
                if (brandSelectable(model)) {
                    brands.add(model.getBrand());
                }
            }
            if (selectedModelNumber == null || customValue.isCustomValue(selectedModelNumber)) {
                brands.add("");
                brands.add(customValue);
            }
            getVisual().setBrands(brands.toArray());
        }

        private boolean brandSelectable(Model model) {
            boolean typeValid = selectedType == null || selectedType.equals(model.getType());
            boolean modelValid = selectedModelNumber == null || customValue.isCustomValue(selectedModelNumber) || selectedModelNumber.equals(model.getModelNumber());

            return typeValid && modelValid;
        }

        private void updateModelNumbers() {
            Set<Object> modelNumbers = new TreeSet<>();
            modelIterator = new ModelIterator();
            while (modelIterator.hasNext()) {
                Model model = modelIterator.next();
                if (modelNumberSelectable(model)) {
                    modelNumbers.add(model.getModelNumber());
                }
            }
            if (modelNumbers.size() > 1) {
                modelNumbers.add("");
            }
            modelNumbers.add(customValue);
            getVisual().setModels(modelNumbers.toArray());
        }

        private boolean modelNumberSelectable(Model model) {
            boolean typeValid = selectedType == null || selectedType.equals(model.getType());
            boolean brandValid = selectedBrand == null || selectedBrand.equals(model.getBrand());

            return typeValid && brandValid;
        }

        private void toggleCustomInputFields() {
            getVisual().enableCustomBrandField(customValue.isCustomValue(selectedBrand));
            getVisual().enableCustomModelNumberField(customValue.isCustomValue(selectedModelNumber));
        }
    }

    private class ModelIterator implements Iterator<Model> {

        private Iterator<Model> iterator;

        public ModelIterator() {
            List<Model> models = new ArrayList<>();
            for (MachineDataService service : getDataServices()) {
                models.addAll(service.getModels());
            }
            iterator = models.iterator();
        }

        @Override
        public boolean hasNext() {
            return iterator.hasNext();
        }

        @Override
        public Model next() {
            return iterator.next();
        }

        @Override
        public void remove() {
            throw new UnsupportedOperationException("Not supported.");
        }

        private Collection<? extends MachineDataService> getDataServices() {
            return Lookup.getDefault().lookupAll(factory.getDataServiceClass());
        }
    }

    private class CustomItemString implements Comparable<Object> {

        @Override
        public String toString() {
            return Bundle.customValue();
        }

        public boolean isCustomValue(Object value) {
            return value != null && value.equals(toString());
        }

        /**
         * Compare this CustomItemString with the given object. The given object
         * is in all cases greater, except for when it is an empty String or
         * null. This way, the CustomItemString is ordered before the "real"
         * items in a SortedSet.
         *
         * @param o The object to compare with
         * @return 1 if the object is an empty string, -1 otherwise
         */
        @Override
        public int compareTo(Object o) {
            if (o == null || o.toString().isEmpty()) {
                return 1;
            }
            return -1;
        }
    }
}
